==> haunted-4.5.0.tgz: package/web.js <==
import { directive, render } from '../lit-html/lit-html.js';
export { html, render } from '../lit-html/lit-html.js';

const symbolFor = typeof Symbol === 'function' ? Symbol.for : str => str;

const phaseSymbol = symbolFor('haunted.phase');
const hookSymbol = symbolFor('haunted.hook');

const updateSymbol = symbolFor('haunted.update');
const commitSymbol = symbolFor('haunted.commit');
const effectsSymbol = symbolFor('haunted.effects');
const contextSymbol = symbolFor('haunted.context');

const contextEvent = 'haunted.context';

let current;
let currentId = 0;

function setCurrent(element) {
  current = element;
}

function clear() {
  current = null;
  currentId = 0;
}

function notify() {
  let id = currentId;
  currentId++;
  return id;
}

//import { render, html } from './lit.js';

const defer = Promise.resolve().then.bind(Promise.resolve());

function scheduler() {
  let tasks = [];
  let id;

  function runTasks() {
    id = null;
    let t = tasks;
    tasks = [];
    for(var i = 0, len = t.length; i < len; i++) {
      t[i]();
    }
  }

  return function(task) {
    tasks.push(task);
    if(id == null) {
      id = defer(runTasks);
    }
  };
}

function makeContainer(render$$1) {
  const read = scheduler();
  const write = scheduler();

  class Container {
    constructor(renderer, frag, host) {
      this.renderer = renderer;
      this.frag = frag;
      this.host = host || frag;
      this[hookSymbol] = new Map();
      this[phaseSymbol] = null;
      this._updateQueued = false;
    }

    update() {
      if(this._updateQueued) return;
      read(() => {
        let result = this.handlePhase(updateSymbol);
        write(() => {
          this.handlePhase(commitSymbol, result);

          if(this[effectsSymbol]) {
            write(() => {
              this.handlePhase(effectsSymbol);
            });
          }
        });
        this._updateQueued = false;
      });
      this._updateQueued = true;
    }

    handlePhase(phase, arg) {
      this[phaseSymbol] = phase;
      switch(phase) {
        case commitSymbol: return this.commit(arg);
        case updateSymbol: return this.render();
        case effectsSymbol: return this.runEffects(effectsSymbol);
      }
      this[phaseSymbol] = null;
    }

    commit(result) {
      render$$1(result, this.frag);
      this.runEffects(commitSymbol);
    }

    render() {
      setCurrent(this);
      let result = this.args ?
        this.renderer.apply(this.host, this.args) :
        this.renderer.call(this.host, this.host);
      clear();
      return result;
    }

    runEffects(symbol) {
      let effects = this[symbol];
      if(effects) {
        setCurrent(this);
        for(let effect of effects) {
          effect.call(this);
        }
        clear();
      }
    }

    teardown() {
      let hooks = this[hookSymbol];
      hooks.forEach((hook) => {
        if (typeof hook.teardown === 'function') {
          hook.teardown();
        }
      });
    }
  }

  return Container;
}

function toCamelCase(val = '') {
  return val.indexOf('-') === -1 ? val.toLowerCase() : val.toLowerCase().split('-').reduce((out, part) => {
    return out ? out + part.charAt(0).toUpperCase() + part.slice(1) : part;
  },'') 
}

function makeComponent(Container) {
  function component(renderer, BaseElement = HTMLElement, {useShadowDOM = true, shadowRootInit = {}} = {}) {
    class Element extends BaseElement {
      static get observedAttributes() {
        return renderer.observedAttributes || [];
      }

      constructor() {
        super();
        if (useShadowDOM === false) {
          this._container = new Container(renderer, this);
        } else {
          this.attachShadow({ mode: "open", ...shadowRootInit});
          this._container = new Container(renderer, this.shadowRoot, this);        
        }
      }

      connectedCallback() {
        this._container.update();
      }

      disconnectedCallback() {
        this._container.teardown();
      }

      attributeChangedCallback(name, _, newValue) {
        let val = newValue === '' ? true : newValue;
        Reflect.set(this, toCamelCase(name), val);
      }
    }
    function reflectiveProp(initialValue) {
      let value = initialValue;
      return Object.freeze({
        enumerable: true,
        configurable: true,
        get() {
          return value;
        },
        set(newValue) {
          value = newValue;
          this._container.update();
        }
      })
    }

    const proto = new Proxy(BaseElement.prototype, {
      set(target, key, value, receiver) {
        if(key in target) {
          Reflect.set(target, key, value);
        }
        let desc;
        if(typeof key === 'symbol' || key[0] === '_') {
          desc = {
            enumerable: true,
            configurable: true,
            writable: true,
            value
          }; 
        } else {
          desc = reflectiveProp(value);
        }
        Object.defineProperty(receiver, key, desc);

        if(desc.set) {
          desc.set.call(receiver, value);
        }

        return true;
      }
    });

    Object.setPrototypeOf(Element.prototype, proto);


    return Element;
  }

  return component;
}

class Hook {
  constructor(id, el) {
    this.id = id;
    this.el = el;
  }
}

function use(Hook, ...args) {
  let id = notify();
  let hooks = current[hookSymbol];
  
  let hook = hooks.get(id);
  if(!hook) {
    hook = new Hook(id, current, ...args);
    hooks.set(id, hook);
  }

  return hook.update(...args);
}

function hook(Hook) {
  return use.bind(null, Hook);
}

function setEffects(el, cb) {
  if(!(effectsSymbol in el)) {
    el[effectsSymbol] = [];
  }
  el[effectsSymbol].push(cb);
}

const useEffect = hook(class extends Hook {
  constructor(id, el) {
    super(id, el);
    this.values = false;
    setEffects(el, this);
  }

  update(callback, values) {
    this.callback = callback;
    this.lastValues = this.values;
    this.values = values;
  }

  call() {
    if(this.values) {
      if(this.hasChanged()) {
        this.run();
      }
    } else {
      this.run();
    }
  }

  run() {
    this.teardown();
    this._teardown = this.callback.call(this.el);
  }

  teardown() {
    if(this._teardown) {
      this._teardown();
    }
  }

  hasChanged() {
    return this.lastValues === false || this.values.some((value, i) => this.lastValues[i] !== value);
  }
});

function setContexts(el, consumer) {
  if(!(contextSymbol in el)) {
    el[contextSymbol] = [];
  }
  el[contextSymbol].push(consumer);
}

const useContext = hook(class extends Hook {
  constructor(id, el) {
    super(id, el);
    setContexts(el, this);
    this._updater = this._updater.bind(this);
    this._ranEffect = false;
    this._unsubscribe = null;
    setEffects(el, this);
  }

  update(Context) {
    if (this.el.virtual) {
      throw new Error('can\'t be used with virtual components');
    }

    if (this.Context !== Context) {
      this._subscribe(Context);
      this.Context = Context;
    }

    return this.value;
  }

  call() {
    if(!this._ranEffect) {
      this._ranEffect = true;
      if(this._unsubscribe) this._unsubscribe();
      this._subscribe(this.Context);
      this.el.update();
    }
  }

  _updater(value) {
    this.value = value;
    this.el.update();
  }

  _subscribe(Context) {
    const detail = { Context, callback: this._updater };

    this.el.host.dispatchEvent(new CustomEvent(contextEvent, {
      detail, // carrier
      bubbles: true, // to bubble up in tree
      cancelable: true, // to be able to cancel
      composed: true, // to pass ShadowDOM boundaries
    }));

    const { unsubscribe, value } = detail;

    this.value = unsubscribe ? value : Context.defaultValue;

    this._unsubscribe = unsubscribe;
  }

  teardown() {
    if (this._unsubscribe) {
      this._unsubscribe();
    }
  }
});

function makeContext(component) {
  return (defaultValue) => {
    const Context = {
      Provider: class extends HTMLElement {
        constructor() {
          super();
          this.listeners = new Set();
      
          this.addEventListener(contextEvent, this);
        }
      
        disconnectedCallback() {
          this.removeEventListener(contextEvent, this);
        }
    
        handleEvent(event) {
          const { detail } = event;
    
          if (detail.Context === Context) {
            detail.value = this.value;
            detail.unsubscribe = this.unsubscribe.bind(this, detail.callback);
    
            this.listeners.add(detail.callback);
    
            event.stopPropagation();
          }
        }
    
        unsubscribe(callback) {
          if(this.listeners.has(callback)) {
            this.listeners.delete(callback);
          }
        }
    
        set value(value) {
          this._value = value;
          for(let callback of this.listeners) {
            callback(value);
          }
        }
    
        get value() {
          return this._value;
        }
      },

      Consumer: component(function ({ render: render$$1 }) {
        const context = useContext(Context);
    
        return render$$1(context);
      }),

      defaultValue
    };

    return Context;
  };
}

const useMemo = hook(class extends Hook {
  constructor(id, el, fn, values) {
    super(id, el);
    this.value = fn();
    this.values = values;
  }

  update(fn, values) {
    if(this.hasChanged(values)) {
      this.values = values;
      this.value = fn();
    }
    return this.value;
  }

  hasChanged(values) {
    return values.some((value, i) => this.values[i] !== value);
  }
});

const useCallback = (fn, inputs) => useMemo(() => fn, inputs);

const useState = hook(class extends Hook {
  constructor(id, el, initialValue) {
    super(id, el);
    this.updater = this.updater.bind(this);

    if(typeof initialValue === 'function') {
      initialValue = initialValue();
    }

    this.makeArgs(initialValue);
  }

  update() {
    return this.args;
  }

  updater(value) {
    if (typeof value === "function") {
      const updaterFn = value;
      const [previousValue] = this.args;
      value = updaterFn(previousValue);
    }

    this.makeArgs(value);
    this.el.update();
  }

  makeArgs(value) {
    this.args = Object.freeze([value, this.updater]);
  }
});

const useReducer = hook(class extends Hook {
  constructor(id, el, _, initialState) {
    super(id, el);
    this.dispatch = this.dispatch.bind(this);
    this.state = initialState;
  }

  update(reducer) {
    this.reducer = reducer;
    return [this.state, this.dispatch];
  }

  dispatch(action) {
    this.state = this.reducer(this.state, action);
    this.el.update();
  }
});

const useRef = (initialValue) => {
    return useMemo(() => {
        return {
            current: initialValue
        };
    }, []);
};

function haunted({ render: render$$1 }) {
  const Container = makeContainer(render$$1);
  const component = makeComponent(Container);
  const createContext = makeContext(component);

  return { Container, component, createContext };
}

const includes = Array.prototype.includes;

function makeVirtual(Container) {
  const partToContainer = new WeakMap();
  const containerToPart = new WeakMap();
  
  class DirectiveContainer extends Container {
    constructor(renderer, part) {
      super(renderer, part);
      this.virtual = true;
    }
  
    commit(result) {
      this.host.setValue(result);
      this.host.commit();
    }
  
    teardown() {
      super.teardown();
      let part = containerToPart.get(this);
      partToContainer.delete(part);
    }
  }
  
  function virtual(renderer) {
    function factory(...args) {
      return part => {
        let cont = partToContainer.get(part);
        if(!cont) {
          cont = new DirectiveContainer(renderer, part);
          partToContainer.set(part, cont);
          containerToPart.set(cont, part);
          teardownOnRemove(cont, part);
        }
        cont.args = args;
        cont.update();
      };
    }
  
    return directive(factory);
  }

  return virtual;
}

function teardownOnRemove(cont, part, node = part.startNode) {
  let frag = node.parentNode;
  let mo = new MutationObserver(mutations => {
    for(let mutation of mutations) {
      if(includes.call(mutation.removedNodes, node)) {
        mo.disconnect();

        if(node.parentNode instanceof ShadowRoot) {
          teardownOnRemove(cont, part);
        } else {
          cont.teardown();
        }
        break;
      } else if(includes.call(mutation.addedNodes, node.nextSibling)) {
        mo.disconnect();
        teardownOnRemove(cont, part, node.nextSibling);
        break;
      }
    }
  });
  mo.observe(frag, { childList: true });
}

const { Container, component, createContext } = haunted({
  render(what, where) {
    render(what, where);
  }
});

const virtual = makeVirtual(Container);

export default haunted;
export { component, createContext, virtual, useCallback, useEffect, useState, useReducer, useMemo, useContext, useRef, hook, Hook };

==> haunted-4.5.2.tgz: package/web.js <==
import { directive, render } from '../lit-html/lit-html.js';
export { html, render } from '../lit-html/lit-html.js';

const symbolFor = typeof Symbol === 'function' ? Symbol.for : str => str;

const phaseSymbol = symbolFor('haunted.phase');
const hookSymbol = symbolFor('haunted.hook');

const updateSymbol = symbolFor('haunted.update');
const commitSymbol = symbolFor('haunted.commit');
const effectsSymbol = symbolFor('haunted.effects');
const contextSymbol = symbolFor('haunted.context');

const contextEvent = 'haunted.context';

let current;
let currentId = 0;

function setCurrent(element) {
  current = element;
}

function clear() {
  current = null;
  currentId = 0;
}

function notify() {
  let id = currentId;
  currentId++;
  return id;
}

//import { render, html } from './lit.js';

const defer = Promise.resolve().then.bind(Promise.resolve());

function scheduler() {
  let tasks = [];
  let id;

  function runTasks() {
    id = null;
    let t = tasks;
    tasks = [];
    for(var i = 0, len = t.length; i < len; i++) {
      t[i]();
    }
  }

  return function(task) {
    tasks.push(task);
    if(id == null) {
      id = defer(runTasks);
    }
  };
}

function makeContainer(render$$1) {
  const read = scheduler();
  const write = scheduler();

  class Container {
    constructor(renderer, frag, host) {
      this.renderer = renderer;
      this.frag = frag;
      this.host = host || frag;
      this[hookSymbol] = new Map();
      this[phaseSymbol] = null;
      this._updateQueued = false;
    }

    update() {
      if(this._updateQueued) return;
      read(() => {
        let result = this.handlePhase(updateSymbol);
        write(() => {
          this.handlePhase(commitSymbol, result);

          if(this[effectsSymbol]) {
            write(() => {
              this.handlePhase(effectsSymbol);
            });
          }
        });
        this._updateQueued = false;
      });
      this._updateQueued = true;
    }

    handlePhase(phase, arg) {
      this[phaseSymbol] = phase;
      switch(phase) {
        case commitSymbol: return this.commit(arg);
        case updateSymbol: return this.render();
        case effectsSymbol: return this.runEffects(effectsSymbol);
      }
      this[phaseSymbol] = null;
    }

    commit(result) {
      render$$1(result, this.frag);
      this.runEffects(commitSymbol);
    }

    render() {
      setCurrent(this);
      let result = this.args ?
        this.renderer.apply(this.host, this.args) :
        this.renderer.call(this.host, this.host);
      clear();
      return result;
    }

    runEffects(symbol) {
      let effects = this[symbol];
      if(effects) {
        setCurrent(this);
        for(let effect of effects) {
          effect.call(this);
        }
        clear();
      }
    }

    teardown() {
      let hooks = this[hookSymbol];
      hooks.forEach((hook) => {
        if (typeof hook.teardown === 'function') {
          hook.teardown();
        }
      });
    }
  }

  return Container;
}

function toCamelCase(val = '') {
  return val.indexOf('-') === -1 ? val.toLowerCase() : val.toLowerCase().split('-').reduce((out, part) => {
    return out ? out + part.charAt(0).toUpperCase() + part.slice(1) : part;
  },'') 
}

function makeComponent(Container) {
  function component(renderer, baseElementOrOptions, options) {
    const BaseElement = (options || baseElementOrOptions || {}).baseElement || HTMLElement;
    const {observedAttributes = [], useShadowDOM = true, shadowRootInit = {}} = options || baseElementOrOptions || {};

    class Element extends BaseElement {
      static get observedAttributes() {
        return renderer.observedAttributes || observedAttributes || [];
      }

      constructor() {
        super();
        if (useShadowDOM === false) {
          this._container = new Container(renderer, this);
        } else {
          this.attachShadow({ mode: "open", ...shadowRootInit});
          this._container = new Container(renderer, this.shadowRoot, this);        
        }
      }

      connectedCallback() {
        this._container.update();
      }

      disconnectedCallback() {
        this._container.teardown();
      }

      attributeChangedCallback(name, _, newValue) {
        let val = newValue === '' ? true : newValue;
        Reflect.set(this, toCamelCase(name), val);
      }
    }
    function reflectiveProp(initialValue) {
      let value = initialValue;
      return Object.freeze({
        enumerable: true,
        configurable: true,
        get() {
          return value;
        },
        set(newValue) {
          value = newValue;
          this._container.update();
        }
      })
    }

    const proto = new Proxy(BaseElement.prototype, {
      set(target, key, value, receiver) {
        if(key in target) {
          Reflect.set(target, key, value);
        }
        let desc;
        if(typeof key === 'symbol' || key[0] === '_') {
          desc = {
            enumerable: true,
            configurable: true,
            writable: true,
            value
          }; 
        } else {
          desc = reflectiveProp(value);
        }
        Object.defineProperty(receiver, key, desc);

        if(desc.set) {
          desc.set.call(receiver, value);
        }

        return true;
      }
    });

    Object.setPrototypeOf(Element.prototype, proto);


    return Element;
  }

  return component;
}

class Hook {
  constructor(id, el) {
    this.id = id;
    this.el = el;
  }
}

function use(Hook, ...args) {
  let id = notify();
  let hooks = current[hookSymbol];
  
  let hook = hooks.get(id);
  if(!hook) {
    hook = new Hook(id, current, ...args);
    hooks.set(id, hook);
  }

  return hook.update(...args);
}

function hook(Hook) {
  return use.bind(null, Hook);
}

function setEffects(el, cb) {
  if(!(effectsSymbol in el)) {
    el[effectsSymbol] = [];
  }
  el[effectsSymbol].push(cb);
}

const useEffect = hook(class extends Hook {
  constructor(id, el) {
    super(id, el);
    this.values = false;
    setEffects(el, this);
  }

  update(callback, values) {
    this.callback = callback;
    this.lastValues = this.values;
    this.values = values;
  }

  call() {
    if(this.values) {
      if(this.hasChanged()) {
        this.run();
      }
    } else {
      this.run();
    }
  }

  run() {
    this.teardown();
    this._teardown = this.callback.call(this.el);
  }

  teardown() {
    if(this._teardown) {
      this._teardown();
    }
  }

  hasChanged() {
    return this.lastValues === false || this.values.some((value, i) => this.lastValues[i] !== value);
  }
});

function setContexts(el, consumer) {
  if(!(contextSymbol in el)) {
    el[contextSymbol] = [];
  }
  el[contextSymbol].push(consumer);
}

const useContext = hook(class extends Hook {
  constructor(id, el) {
    super(id, el);
    setContexts(el, this);
    this._updater = this._updater.bind(this);
    this._ranEffect = false;
    this._unsubscribe = null;
    setEffects(el, this);
  }

  update(Context) {
    if (this.el.virtual) {
      throw new Error('can\'t be used with virtual components');
    }

    if (this.Context !== Context) {
      this._subscribe(Context);
      this.Context = Context;
    }

    return this.value;
  }

  call() {
    if(!this._ranEffect) {
      this._ranEffect = true;
      if(this._unsubscribe) this._unsubscribe();
      this._subscribe(this.Context);
      this.el.update();
    }
  }

  _updater(value) {
    this.value = value;
    this.el.update();
  }

  _subscribe(Context) {
    const detail = { Context, callback: this._updater };

    this.el.host.dispatchEvent(new CustomEvent(contextEvent, {
      detail, // carrier
      bubbles: true, // to bubble up in tree
      cancelable: true, // to be able to cancel
      composed: true, // to pass ShadowDOM boundaries
    }));

    const { unsubscribe, value } = detail;

    this.value = unsubscribe ? value : Context.defaultValue;

    this._unsubscribe = unsubscribe;
  }

  teardown() {
    if (this._unsubscribe) {
      this._unsubscribe();
    }
  }
});

function makeContext(component) {
  return (defaultValue) => {
    const Context = {
      Provider: class extends HTMLElement {
        constructor() {
          super();
          this.listeners = new Set();
      
          this.addEventListener(contextEvent, this);
        }
      
        disconnectedCallback() {
          this.removeEventListener(contextEvent, this);
        }
    
        handleEvent(event) {
          const { detail } = event;
    
          if (detail.Context === Context) {
            detail.value = this.value;
            detail.unsubscribe = this.unsubscribe.bind(this, detail.callback);
    
            this.listeners.add(detail.callback);
    
            event.stopPropagation();
          }
        }
    
        unsubscribe(callback) {
          if(this.listeners.has(callback)) {
            this.listeners.delete(callback);
          }
        }
    
        set value(value) {
          this._value = value;
          for(let callback of this.listeners) {
            callback(value);
          }
        }
    
        get value() {
          return this._value;
        }
      },

      Consumer: component(function ({ render: render$$1 }) {
        const context = useContext(Context);
    
        return render$$1(context);
      }),

      defaultValue
    };

    return Context;
  };
}

const useMemo = hook(class extends Hook {
  constructor(id, el, fn, values) {
    super(id, el);
    this.value = fn();
    this.values = values;
  }

  update(fn, values) {
    if(this.hasChanged(values)) {
      this.values = values;
      this.value = fn();
    }
    return this.value;
  }

  hasChanged(values) {
    return values.some((value, i) => this.values[i] !== value);
  }
});

const useCallback = (fn, inputs) => useMemo(() => fn, inputs);

const useState = hook(class extends Hook {
  constructor(id, el, initialValue) {
    super(id, el);
    this.updater = this.updater.bind(this);

    if(typeof initialValue === 'function') {
      initialValue = initialValue();
    }

    this.makeArgs(initialValue);
  }

  update() {
    return this.args;
  }

  updater(value) {
    if (typeof value === "function") {
      const updaterFn = value;
      const [previousValue] = this.args;
      value = updaterFn(previousValue);
    }

    this.makeArgs(value);
    this.el.update();
  }

  makeArgs(value) {
    this.args = Object.freeze([value, this.updater]);
  }
});

const useReducer = hook(class extends Hook {
  constructor(id, el, _, initialState) {
    super(id, el);
    this.dispatch = this.dispatch.bind(this);
    this.state = initialState;
  }

  update(reducer) {
    this.reducer = reducer;
    return [this.state, this.dispatch];
  }

  dispatch(action) {
    this.state = this.reducer(this.state, action);
    this.el.update();
  }
});

const useRef = (initialValue) => {
    return useMemo(() => {
        return {
            current: initialValue
        };
    }, []);
};

function haunted({ render: render$$1 }) {
  const Container = makeContainer(render$$1);
  const component = makeComponent(Container);
  const createContext = makeContext(component);

  return { Container, component, createContext };
}

const includes = Array.prototype.includes;

function makeVirtual(Container) {
  const partToContainer = new WeakMap();
  const containerToPart = new WeakMap();
  
  class DirectiveContainer extends Container {
    constructor(renderer, part) {
      super(renderer, part);
      this.virtual = true;
    }
  
    commit(result) {
      this.host.setValue(result);
      this.host.commit();
    }
  
    teardown() {
      super.teardown();
      let part = containerToPart.get(this);
      partToContainer.delete(part);
    }
  }
  
  function virtual(renderer) {
    function factory(...args) {
      return part => {
        let cont = partToContainer.get(part);
        if(!cont) {
          cont = new DirectiveContainer(renderer, part);
          partToContainer.set(part, cont);
          containerToPart.set(cont, part);
          teardownOnRemove(cont, part);
        }
        cont.args = args;
        cont.update();
      };
    }
  
    return directive(factory);
  }

  return virtual;
}

function teardownOnRemove(cont, part, node = part.startNode) {
  let frag = node.parentNode;
  let mo = new MutationObserver(mutations => {
    for(let mutation of mutations) {
      if(includes.call(mutation.removedNodes, node)) {
        mo.disconnect();

        if(node.parentNode instanceof ShadowRoot) {
          teardownOnRemove(cont, part);
        } else {
          cont.teardown();
        }
        break;
      } else if(includes.call(mutation.addedNodes, node.nextSibling)) {
        mo.disconnect();
        teardownOnRemove(cont, part, node.nextSibling);
        break;
      }
    }
  });
  mo.observe(frag, { childList: true });
}

const { Container, component, createContext } = haunted({
  render(what, where) {
    render(what, where);
  }
});

const virtual = makeVirtual(Container);

export default haunted;
export { component, createContext, virtual, useCallback, useEffect, useState, useReducer, useMemo, useContext, useRef, hook, Hook };
